/*
 * (C) IBM Corp. 1997-1998
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.parser;

import java.io.*;
import java.util.StringTokenizer;

/**
 * XML Syntax Checker.
 *
 * @version 19980206
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class trlx {
    private trlx() {
    }
    
/*
    static class AElementFactory extends DefaultElementFactory {
        int count = 0;
        int tcount = 0;
        public Element createElement(String qname, String ns, String n) {
            count ++;
            return new Element(qname, ns, n);
        }
        public TextElement createTextElement(String data) {
            tcount ++;
            return new TextElement(data);
        }
        void print() {
            System.out.println("Element: "+count);
            System.out.println("TextElement: "+tcount);
        }
    }
*/
    private static void reportMemory(Runtime rt) {
        long total = rt.totalMemory();
        long free = rt.freeMemory();
        long using = total-free;
        System.err.println(free+"/"+total+": use "+using+" Byte");
    }

    public static void main(String[] argv) {
        try {
            String fname = null;
            boolean isprint = false;
            String encname = null;
            boolean isprintclass = false;
            boolean isallowj = false;
            boolean iskeepcomment = false;
            boolean reportmemory = false;
            boolean donamespace = false;
            boolean warnnoxml = false;
            Runtime rt = Runtime.getRuntime();
            int loop = 0;
            for (int i = 0;  i < argv.length;  i ++) {
                if ('-' != argv[i].charAt(0)) {
                    fname = argv[i];

                } else if ("-memory".equals(argv[i])) {
                    reportmemory = true;
                    System.err.print("Initial free/total memory:\n\t");
                    reportMemory(rt);

                } else if ("-print".equals(argv[i]) || "-d".equals(argv[i])) {
                    isprint = true;
                } else if ("-class".equals(argv[i])) {
                    isprintclass = true;

                } else if ("-n".equals(argv[i]) || "-t".equals(argv[i])) {
                    if (i+1 < argv.length) {
                        int v = Integer.parseInt(argv[++i]);
                        if (0 >= v) {
                            System.err.println("-n/-t requires a number more than 0.");
                        } else
                            loop = v;
                    } else {
                        System.err.println("-n/-t requires a number.");
                    }

                } else if ("-outencoding".equals(argv[i])) {
                    if (i+1 < argv.length) {
                        encname = argv[++i];
                    } else {
                        System.err.println("-outencoding requires an encoding name.");
                    }

                } else if ("-allowjavaencoding".equals(argv[i])) {
                    isallowj = true;

                } else if ("-namespace".equals(argv[i])) {
                    donamespace = true;

                } else if ("-keepcomment".equals(argv[i])) {
                    iskeepcomment = true;

                } else if ("-warnnoxml".equals(argv[i])) {
                    warnnoxml = true;

                } else {
                    System.err.println("Warning: Unknown option: "+argv[i]);
                }
            }

            InputStream is;
            TXDocument doc = null;
            if (0 == loop) {
                if (null == fname) {
                    System.err.println("XML Syntax Checker");
                    System.err.println("Usage:\n\tjava trlx [options] <URL or filename>");
                    System.err.println("Options:\n\t-d\tPrint parsed object in XML format");
                    System.err.println("\t-outencoding mimename\tSpecify encoding when print");
                    System.err.println("\t-namespace\tEnable namespace feature");
                    System.err.println("\t-n <n>\tParse n times and print time");
                    System.err.println("\t-memory\tReport memory usage");
                    System.err.println("\tTo know other options, See the source :-)");
                    System.exit(1);
                }
                Stderr se = new Stderr(fname);
                is = se.getInputStream(fname, null, fname);

                Parser pc = new Parser(fname, se, se);
                /*
                AElementFactory ef = new AElementFactory();
                pc.setElementFactory(ef);
                */
                pc.setAllowJavaEncodingName(isallowj);
                pc.setKeepComment(iskeepcomment);
                pc.setWarningNoDoctypeDecl(false);
                pc.setProcessNamespace(donamespace);
                pc.setWarningNoXMLDecl(warnnoxml);
                doc = pc.readStream(is);
                is.close();
                if (null == doc) {
                    System.err.println("No document");
                } else if (isprint) {
                    if (null == encname) {
                        doc.print(new PrintWriter(new OutputStreamWriter(System.out)));
                    } else {
                        String enc = MIME2Java.convert(encname);
                        if (null == enc) {
                            System.err.println("Encoding `"+encname+"' is not supported.");
                            doc.print(new PrintWriter(new OutputStreamWriter(System.out)));
                        } else {
                            doc.print(new PrintWriter(new OutputStreamWriter(System.out, enc)),
                                      enc);
                        }

                    }
                }
                /*
                ef.print();
                */
                if (isprintclass)
                    printClassTree(doc, new PrintWriter(new OutputStreamWriter(System.out)), 0);

            } else {
                long total = 0;
                for (int i = 0;  i < loop;  i ++) {
                    long t = System.currentTimeMillis();
                    Parser pc = new Parser(fname);
                    pc.setAllowJavaEncodingName(isallowj);
                    pc.setKeepComment(iskeepcomment);
                    pc.setWarningNoDoctypeDecl(false);
                    pc.setProcessNamespace(donamespace);
                    pc.setWarningNoXMLDecl(warnnoxml);
                    doc = pc.readStream(is = new FileInputStream(fname));
                    is.close();
                    t = System.currentTimeMillis()-t;
                    System.err.println(t+" [msec]");
                    total += t;
                }
                System.err.println("Total: "+total+" [msec],  Average: "+(total/loop)+" [msec]");
            }
            if (reportmemory) {
                System.err.print("Free/total memory after processing:\n\t");
                reportMemory(rt);
                System.err.println("Garbage Collecting ...");
                rt.gc();
                System.err.print("Free/total memory after gc:\n\t");
                reportMemory(rt);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    static String getLastName(String cl) {
        String ret = cl;
        StringTokenizer st = new StringTokenizer(cl, " .");
        while (st.hasMoreTokens())
            ret = st.nextToken();
        return ret;
    }
    static void printClassTree(Parent par, PrintWriter pw, int indent) {
        org.w3c.dom.NodeEnumerator ne = par.getChildren().getEnumerator();
        for (Child ch = (Child)ne.getFirst();  null != ch;  ch = (Child)ne.getNext()) {
            Child.printSpace(pw, indent);
            pw.println(getLastName(ch.getClass().toString())
                       +" \""+TXText.makePrintable(ch.getText())+"\"");
            if (ch instanceof Parent) {
                printClassTree((Parent)ch, pw, indent+2);
            }
        }
        pw.flush();
    }
}
